#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* the types of commands the droid follows */
typedef enum {
    PUSH,
    ADD,
    SUB,
    MOVE,
    TURN,
    BRANCH,
    HALT
} Type;


/* a single droid command */
typedef struct {
    Type type;
    int value;
} Inst;

/* for debugging porpoises */
void print_inst(Inst* inst) {
    switch (inst->type) {
        case PUSH: printf("PUSH %d\n", inst->value); break;
        case ADD: printf("ADD\n"); break;
        case MOVE: printf("MOVE\n"); break;
        case TURN: printf("TURN\n"); break;
        case BRANCH: printf("BRANCH %d\n", inst->value); break;
        case HALT: printf("HALT\n"); break;
    }
}

/* the state of the lil droid */
typedef struct {
    /* the x y position */
    int x, y;

    /* the direction, essentially
     * 1 means increase x/y, -1 means decrease, 0 means not at all */
    int dx, dy;

    /* which line of the control program he's on */
    int line;

    /* the stack and top index */
    int stack[100];
    int top;

} Droid;

/* a constructor for it */
Inst inst(Type t, int value) {
    Inst inst = {t, value};
    return inst;
}

/* reads in the control program */
void read_program(Inst* program, int* lines) {
    /* read the number of lines */
    scanf("%d\n", lines);

    /* read the lines */
    int i;
    for (i = 0; i < *lines; i++) {
        char line[100];
        scanf("%[^\n]\n", line);

        /* get the parameter if any */
        char* space = strstr(line, " ");
        int param = 0;
        if (space) {
            space++;
            param = atoi(space);
        }

        /* switch on the type of instruction it is */
        switch (line[0]) {
            case 'p': program[i] = inst(PUSH, param); break;
            case 'a': program[i] = inst(ADD, 0); break;
            case 'm': program[i] = inst(MOVE, 0); break;
            case 't': program[i] = inst(TURN, 0); break;
            case 'b': program[i] = inst(BRANCH, param); break;
            case 'h': program[i] = inst(HALT, 0); break;
        }
    }
}

/* handles rotating the droid in place by some angle */
void handleTurn(Droid* droid, int angle) {
    /* first we get the angle to be in the range [0 - 270] it must be a
     * multiple of 90, so this gives us just 4 possibilities */
    while (angle < 0)
        angle += 360;
    while (angle >= 360)
        angle -= 360;

    /* now we handle each of the four cases */
    switch (angle) {
        case 0:
            /* do nothing lol */
            break;

        case 90:
            /* handle the four cases of which way we are facing now */
            if(droid->dy == 1) {
                droid->dx = -1;
                droid->dy = 0;
            } else if(droid->dy == -1) {
                droid->dx = 1;
                droid->dy = 0;
            } else if(droid->dx == 1) {
                droid->dx = 0;
                droid->dy = 1;
            } else if(droid->dx == -1) {
                droid->dx = 0;
                droid->dy = -1;
            }
            break;

        case 180:
            /* just about face */
            droid->dx *= -1;
            droid->dy *= -1;
            break;

        case 270:
            /* for this one, we just rotate by 180, then 90 yay recursion */
            handleTurn(droid, 180);
            handleTurn(droid, 90);
            break;
    }
}

/* simulate 1 line of the droid on this control program
 * returns 0 on halt, 1 on keep going */
int simulate(Droid* droid, Inst* program) {
    int i, j;

    switch (program[droid->line].type) {
        /* push the value on the stack and move on */
        case PUSH:
            droid->top++;
            droid->stack[droid->top] = program[droid->line].value;
            droid->line++;
            break;

        /* pop two values, and push result, then move on */
        case ADD:
            i = droid->stack[droid->top--];
            j = droid->stack[droid->top];
            droid->stack[droid->top] = i + j;
            droid->line++;
            break;

        /* pop and move "forward" that amound where "forward"
         * is determined by heading */
        case MOVE:
            i = droid->stack[droid->top--];
            droid->x += droid->dx * i;
            droid->y += droid->dy * i;
            droid->line++;
            break;

        /* this is the complicated one */
        case TURN:
            i = droid->stack[droid->top--];
            handleTurn(droid, i);
            droid->line++; 
            break;

        /* if the top of the stack is not zero, set line number to val in code
         * else just move to next line of program */
        case BRANCH:
            i = droid->stack[droid->top];
            if (i != 0)
                droid->line = program[droid->line].value;
            else
                droid->line++;
            break;

        /* that's a wrap */
        case HALT:
            return 0;
    }

    return 1;
}

int main() {
    Inst program[100];
    int lines;

    /* read the program */
    read_program(program, &lines);

    /* init the droid */
    Droid droid;
    droid.x = 0;
    droid.y = 0;
    droid.dx = 0;
    droid.dy = 1;
    droid.line = 0;
    droid.top = -1;

    /* keep running the program until it halts */
    while (simulate(&droid, program));

    /* do the output based on droid state */
    if (droid.x && droid.y)
        printf("The droid halts %d meters %s and %d meters %s.\n", abs(droid.x),
                droid.x > 0 ? "right" : "left", abs(droid.y), droid.y > 0 ? "forward" : "backward");
    else if (droid.x)
        printf("The droid halts %d meters %s.\n", abs(droid.x), droid.x > 0 ? "right" : "left");
    else if (droid.y)
        printf("The droid halts %d meters %s.\n", abs(droid.y), droid.y > 0 ? "forward" : "backward");
    else
        printf("The droid halts where it began.\n");
    

    return 0;
}

