#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define MAX_STRING 32
#define MAX_CANDIDATES 32
#define MAX_BALLOTS 1000

typedef struct {
    char s[MAX_STRING];
} String;

typedef struct {
    int votes[MAX_CANDIDATES];
    int count;
} Ballot;

/* checks if a ballot is spoiled (those that vote for one candidate twice, or
 * have a non-existent candidate) */
int is_spoiled(Ballot* ballot, int candidate_count) {
    int i;

    /* keep track of which candidate numbers have been used */
    int used[MAX_CANDIDATES];
    for (i = 0; i < MAX_CANDIDATES; i++) {
        used[i] = 0;
    }

    /* for each entry on it */
    for (i = 0; i < ballot->count; i++) {
        /* if the entry is over the limit, it's spoiled */
        if (ballot->votes[i] > candidate_count) {
            return 1;
        }

        /* if the candidate has been used, it's spoiled */
        if (used[ballot->votes[i] - 1]) {
            return 1;
        }

        /* mark as used */
        used[ballot->votes[i] - 1] = 1;
    }

    /* if we got through them all, it's not spoiled */
    return 0;
}

/* remove spoiled ballots  */
int remove_spoiled(Ballot* ballots, int* ballot_count, int candidate_count) {
    int i, j;

    /* for each ballot */
    for (i = 0; i < *ballot_count; i++) {
        if (is_spoiled(&ballots[i], candidate_count)) {
            /* if it's the last ballot */
            if (i == (*ballot_count - 1)) {
                /* just decrement ballot count */
                *ballot_count -= 1;
            } else {
                /* move last one over top of this one */
                ballots[i] = ballots[*ballot_count - 1];

                /* decrement */
                *ballot_count -= 1;

                /* decrement i so THIS one gets checked next time */
                i--;
            }
        }
    }
}

/* checks if there is currently a winner and returns index if so, or -1 */
int check_winner(String* candidates, int candidate_count, Ballot* ballots, int ballot_count) {
    int i, winner;

    /* store 1st place votes for each candidate */
    int* votes = malloc(candidate_count * sizeof(int));
    for (i = 0; i < candidate_count; i++) {
        votes[i] = 0;
    }

    /* go through each ballot */
    for (i = 0; i < ballot_count; i++) {
        /* if they have votes left */
        if (ballots[i].count > 0) {
            /* add a tally for their first choice (-1 is 1 to 0 indexing) */
            votes[ballots[i].votes[0] - 1] += 1;
        }
    }

    /* now loop through and see if any candidate has more than half */
    winner = -1;
    for (i = 0; i < candidate_count; i++) {
        if (votes[i] > (ballot_count / 2)) {
            winner = i;
            break;
        }
    }

    /* clean up and return */
    free(votes); 
    return winner;
}

/* return a copy of a ballot without a candidate on it */
Ballot patch(Ballot ballot, int candidate_number) {
    int i;
    Ballot new;
    new.count = 0;

    /* for each vote on the original */
    for (i = 0; i < ballot.count; i++) {
        /* copy if not for the removed candidate */
        if (ballot.votes[i] != candidate_number) {
            new.votes[new.count++] = ballot.votes[i];
        }
    }

    return new;
}

/* remove one single candidate number from a set of ballots */
void remove_candidate(String* candidates, Ballot* ballots, int ballot_count, int candidate_number) {
    int i;

    /* for each ballot */
    for (i = 0; i < ballot_count; i++) {
        /* set the ballot to be the patched version - without this candidate */
        ballots[i] = patch(ballots[i], candidate_number);
    }

    /* remove the candidates name from the list of candidates as well */
    strcpy(candidates[candidate_number - 1].s, "");
}

/* remove candidates with the lowest number of first place votes need to also
 * fix ballots to remove votes for them */
void remove_candidates(String* candidates, int candidate_count, Ballot* ballots, int ballot_count, int* running) {
    int i;

    /* count the number of 1st place votes for each candidate */
    int* first_places_votes = malloc(candidate_count * sizeof(int));
    for (i = 0; i < candidate_count; i++) {
        first_places_votes[i] = 0;
    }

    /* for each ballot */
    for (i = 0; i < ballot_count; i++) {
        /* add a 1st place vote for who they voted first */
        if (ballots[i].count > 0) {
            first_places_votes[ballots[i].votes[0] - 1] += 1;
        }
    }

    /* now find the minimum number of first place votes */
    int min_votes = ballot_count;
    for (i = 0; i < candidate_count; i++) {
        if (first_places_votes[i] < min_votes && (strlen(candidates[i].s) > 0)) {
            min_votes = first_places_votes[i];
        }
    }

    /* remove all of the candidates with the fewest votes */
    for (i = 0; i < candidate_count; i++) {
        if (first_places_votes[i] == min_votes) {
            printf("%s with %d votes is eliminated.\n", candidates[i].s, min_votes);
            remove_candidate(candidates, ballots, ballot_count, i + 1);
            *running -= 1;
        }
    }

    /* clean up */
    free(first_places_votes);
}

void simulate(String* candidates, int candidate_count, Ballot* ballots, int ballot_count) {
    /* keep track of how may people are still in it */
    int running = candidate_count;

    while (running > 0) {
        /* if there is a winner, declare them */
        int winner = check_winner(candidates, candidate_count, ballots, ballot_count);
        if (winner != -1) {
            printf("The winner of the election is %s.\n", candidates[winner].s);
            return;
        }

        /* remove the candidate(s) who fail to make the cut */
        remove_candidates(candidates, candidate_count, ballots, ballot_count, &running);
    }
}

int main() {
    String candidates[MAX_CANDIDATES];
    Ballot ballots[MAX_BALLOTS];

    int candidate_count, ballot_count, i, tmp;
    scanf("%d %d\n", &candidate_count, &ballot_count);

    /* read candidates */
    for (i = 0; i < candidate_count; i++) {
        scanf("%d %[^\n]\n", &tmp, candidates[i].s);
    }

    /* read ballots */
    for (i = 0; i < ballot_count; i++) {
        char temp[1000], *t;
        scanf("%[^\n]\n", temp);
        int n = atoi(strtok(temp, " "));
        ballots[i].votes[ballots[i].count++] = n;
        while ((t = strtok(NULL, " ")) != NULL) {
            n = atoi(t);
            ballots[i].votes[ballots[i].count++] = n;
        }
    }

    /* remove the spoiled ballots */
    remove_spoiled(ballots, &ballot_count, candidate_count);

    /* simulate the election */
    simulate(candidates, candidate_count, ballots, ballot_count);
    return 0;
}

